/**
   Implements a very simple linked list, each link points to item 
   of type T and references the next link. O(1) operations are 
   getting the first link's data or the second's link reference, 
   or deleting the first element. Each functions modifying the 
   linking returns a pointer to the first link of the list.
   
 */

public class LinkedList<T>
{
    /************************************************/

    private T data;

    /************************************************/

    private LinkedList<T> next;
    
    /************************************************/

    /**
       Creates a one element list.
     */
    
    public LinkedList(T data)
    {
	this.data = data;
    }
    
    /************************************************/

    /**
       Creates a one link list and appends the second 
       paramter to this list.
     */

    public LinkedList(T data, LinkedList<T> next)
    {
	this(data);
	this.next = next;
    }
    
    /************************************************/

    /**
       Modifies the data.
     */

    public void setData(T data)
    {
	this.data = data;
    }
    
    /************************************************/

    /**
       Appends a list to the first link of the current 
       list.
     */

    public void setNext(LinkedList<T> next)
    {
	this.next = next;
    }

    /************************************************/

    /**
       Retrives the data of the first link. 
     */
    
    public T getData()
    {
	return data;
    }
    
    /************************************************/

    /**
       Returns the second item of the list, doesn't change 
       the linking
     */

    public LinkedList<T> getNext()
    {
	return next;
    }
    
    /************************************************/

    /**
       Delete from this list the link which the adress is l, 
       returns the list whithout l.
     */

    public LinkedList<T> deleteLink(LinkedList<T> l)
    {
	if (this == l)
	    return getNext();
	setNext(getNext().deleteLink(l));
	return this;
    }
    
    /************************************************/

    /**
       Returns a String representation of the list.
     */

    public String toString()
    {
	return " -> " + data + 
	    ((next != null) ? next.toString() : "X");
    }

    /************************************************/

    private static <K> LinkedList<K> reverseAcc(LinkedList<K> link, 
					    LinkedList<K> acc)  
    {
	if (link == null)
	    return acc;
	LinkedList<K> next = link.getNext();
	link.setNext(acc);
	return reverseAcc(next, link);
    }

    /************************************************/

    /**
       Adds the link l in front of this list, returns 
       the list thus created.
     */

    public LinkedList<T> addFront(LinkedList<T> l)
    {
	l.setNext(this);
	return l;
    }
    
    /************************************************/

    /**
       Adds the item o in front of this list, returns 
       the new list.
     */
    
    public LinkedList<T> addFront(T o)
    {
	return addFront(new LinkedList<T>(o));
    }

    /************************************************/

    /**
       Reverse the order of the items of the list. Returns the 
       new list.
     */
    
    public LinkedList<T> reverse()
    {
	return reverseAcc(this, null);
    }
    
} 
