#ifndef AVL_H

#include"linkedList.h"

#define AVL_H

typedef struct
{
  /*
    Pointeur vers une fonction permettant de 
    recuperer la cle de chaque donnee.  
   */
  int (*getKey)(void*);

  /*
    Pointeur vers une fonction permettant de 
    de detruire chaque donnee.  
  */
  void (*freeData)(void*);
       
  /*
   Pointeur vers la racine de l'arbre.
  */
  void* root;
}avl ;

/*------------------------------------------------------*/

/*
  Retourne un AVL vide.
*/

avl* avlCreate(int (*getKey)(void*), void (*freeData)(void*));

/*
  Modifie la fonction de destruction des donnees.
*/

void avlSetFreeFunction(avl* a, void (*freeData)(void*));

/*
  Affiche les cles de l'AVL a dans l'ordre croissant, O(n).
*/

void avlPrintKeys(avl* a);

/*
  Insere la donnee v dans l'AVL a, O(log n).
*/

void avlInsert(avl* a, void* v);

/*
  Retourne la donnee de cle x si elle se trouve dans l'AVL a,
  NULL sinon, O(log n).
*/

void* avlFind(avl* a, int x);

/*
  Supprime le noeud de cle k de l'AVL a, applique la fonction 
  de destruction a la donnee de cle k, O(log n).
*/

void avlRemove(avl* a, int k);

/*
  Detruit l'AVL a, applique la fonction de destruction a 
  toutes les donnees du sous-arbre, O(n).
*/

void avlDestroy(avl* a);

/*
  Retourne une liste chainee contenant toutes les donnees
  de l'AVL a disposees dans l'ordre croissant, O(n).
 */

linkedList* avlToList(avl* a);

#endif
