#include"bTree.h"

/************************************************/

bTree* createBT(bTree* left, int data, bTree* right)
{
  bTree* root = (bTree*)malloc(sizeof(bTree));
  if (root == NULL)
    {
      printf("heap overflow...\n");
      exit(0);
    }
  root->left = left;
  root->data = data;
  root->right = right;
  return root;
}

/************************************************/

void destroyBT(bTree* root)
{
  if (root != NULL)
    {
      destroyBT(root->left);
      destroyBT(root->right);
      free(root);
    }
}

/************************************************/

void printBT(bTree* root)
{
  if (root != NULL)
    {
      printf("("); 
      printBT(root->left);
      printf(", %d, ", root->data); 
      printBT(root->right);
      printf(")"); 
    }
  else
    printf("X");
}

/************************************************/

/*
*/

ll* llOfBTAux(bTree* root, ll* acc)
{
  if (root == NULL)
    return acc;
  return llOfBTAux(root->left, 
		   createLL(root->data, llOfBTAux(root->right, acc))
		   );
}

/************************************************/

ll* llOfBT(bTree* root)
{
  return llOfBTAux(root, NULL);
}

/************************************************/

/*
*/

ll* distinctllOfBTAux(bTree* root, ll* acc)
{
  if (root == NULL)
    return acc;
  if (!xInLL(acc, root->data))
    acc = createLL(root->data, acc);
  return distinctllOfBTAux(root->right, 
			   distinctllOfBTAux(root->left, acc));
}

/************************************************/

ll* distinctllOfBT(bTree* root)
{
  return distinctllOfBTAux(root, NULL);
}

/************************************************/

bTree* randomBT(int depht)
{
  if (depht < 0)
    return NULL;
  return createBT(randomBT(depht - 1), 
		  rand()%MOD, 
		  randomBT(depht - 1));
}

/************************************************/

int heightBT(bTree* root)
{
  if (root == NULL)
    return -1;
  else
    return 1 + 
      MAX(heightBT(root->left), heightBT(root->right));
}

/************************************************/

int sumBT(bTree* root)
{
  if (root == NULL)
    return 0;
  else
    return root->data 
      + sumBT(root->left) + sumBT(root->right);
}

/************************************************/

int findBT(bTree* root, int x)
{
  if (root == NULL)
    return 0;
  return (root->data == x) 
    || findBT(root->left, x) || findBT(root->right, x);
}

/************************************************/

int nbXBT(bTree* root, int x)
{
  if (root == NULL)
    return 0;
  return (root->data == x) 
    + nbXBT(root->left, x) + nbXBT(root->right, x);
}

/************************************************/

bTree* copyBT(bTree* root)
{
  if (root == NULL)
    return NULL;
  return createBT(copyBT(root->left), 
		  root->data, copyBT(root->right));
}

/************************************************/

int isALeaf(bTree* root)
{
  return (root != NULL && 
	  root->left == NULL && root->right == NULL);
}

/************************************************/

int maxBT(bTree* root)
{
  int max;
  if (root == NULL)
    {
      printf("function maxBT can't receive an empty tree\n");
      return -1;
    }
  max = root->data;
  if (root->left != NULL)
    max = MAX (max, maxBT(root->left));
  if (root->right != NULL)
    max = MAX (max, maxBT(root->right));
  return max;
}

/************************************************/

bTree* destroyLeavesBT(bTree* root)
{
  if (root == NULL)
    return NULL;
  if (isALeaf(root))
    {
      destroyBT(root);
      return NULL;
    }
  root->left = destroyLeavesBT(root->left);
  root->right = destroyLeavesBT(root->right);
  return root;
}

/************************************************/

/*
*/

int powInt(int b, int n)
{
  if (n == 0)
    return 1;
  return b * powInt(b, n - 1);
}

/************************************************/

/*
*/

bTree* integersBTAux(int n, int k)
{
  if (n < 0)
    return NULL;
  int root = powInt(2, n) + k;
  return createBT(
		  integersBTAux(n - 1, k), 
		  root,
		  integersBTAux(n - 1, root));
}

/************************************************/

bTree* integersBT(int n)
{
  return integersBTAux(n, 0);
}

/************************************************/

/*
*/

bTree* destroyRootBTAux(bTree* root)
{
  if (root == NULL || isALeaf(root))
    return NULL;
  if (root->left != NULL)
    {
      bTree* leftSon = root->left;
      root->left = destroyRootBTAux(root->left);
      leftSon->right = root->right;
      leftSon->left = root->left;
      return leftSon;
    }
  root->left = root->right;
  root->right = NULL;
  return destroyRootBTAux(root);
}

/************************************************/

bTree* destroyRootBT(bTree* root)
{
  if (root == NULL)
    return NULL;
  bTree* res = destroyRootBTAux(root);
  free(root);
  return res;
}

/************************************************/

int maxDephtBT(bTree* root, int x)
{
  if (root == NULL)
    return -1;
  int k = (root->data == x) ? 0 : -1;
  int rec = MAX(maxDephtBT(root->left, x), maxDephtBT(root->right, x));
  return (rec >= 0) ? rec + 1  : k;
}

/************************************************/

bTree* deleteXBT(bTree* root, int x)
{
  if (root == NULL)
    return NULL;
  root->left = deleteXBT(root->left, x);
  root->right = deleteXBT(root->right, x);
  return (root->data == x) ? destroyRootBT(root) : root;
}

/************************************************/

bTree* purgeBT(bTree* root)
{
  if (root == NULL)
    return NULL;
  root->left = purgeBT(deleteXBT(root->left, root->data));
  root->right = purgeBT(deleteXBT(root->right, root->data));
  return root;
}

/************************************************/

ll* findPathBT(bTree* root, int x)
{
  ll* l;
  if (root == NULL)
    return NULL;
  if (root->data == x)
    return createLL(x, NULL);
  l = findPathBT(root->left, x);
  if (l != NULL)
    return createLL(root->data, l);
  l = findPathBT(root->right, x);
  if (l != NULL)
    return createLL(root->data, l);
  return NULL;
}

