#include"genericTree.h"

/*-------------------------------------------------------------*/

void* easyMalloc(unsigned int size)
{
  void* a = malloc(size);
  if (a == NULL)
    {
      printf("easyMalloc : plus de memoire.");
      exit(1);
    }
  return a;
}

/*-------------------------------------------------------------*/

abP abMake(void* data, abP left, abP right)
{
  abP a = (abP)easyMalloc(sizeof(ab));
  a->data = data;
  a->left = left;
  a->right = right;
  return a;
}

/*-------------------------------------------------------------*/

void* abAgregate(abP a, 
		 void* (*f)(void* data, 
			    void* leftImage, 
			    void* rightImage))
{
  if (a == NULL)
      return f(NULL, NULL, NULL);
  return f(a->data, 
	   abAgregate(a->left, f), 
	   abAgregate(a->right, f));
}

/*-------------------------------------------------------------*/

abP abMap(abP a, void* (*f)(void*))
{
  void* mapFunction(void* data, void* left, void* right)
    {
      return abMake(f(data), left, right);
    }
  return abAgregate(a, mapFunction);
}

/*-------------------------------------------------------------*/

void abInfixApply(abP a, void (*f)(void*))
{
  if (a != NULL)
    {
      abInfixApply(a->left, f);
      f(a->data);
      abInfixApply(a->right, f);
    }
}

/*-------------------------------------------------------------*/

void abDestroy(abP a, void (*destroyData)(void*))
{
  if (a != NULL)
    {
      abDestroy(a->left, destroyData);
      abDestroy(a->right, destroyData);
      destroyData(a->data);
      free(a);
    }
}

/*-------------------------------------------------------------*/

unsigned int abHeight(abP a)
{
  unsigned int *i;
  unsigned int res;
  void* f(void* data, void* left, void* right)
    {
      unsigned int* i = (unsigned int*)easyMalloc(sizeof(unsigned int));
      if (data == NULL)
	*i = 0;
      else
	*i = (*(unsigned int *)left > *(unsigned int *)right) ? 
	  *(unsigned int *)left : *(unsigned int *)right;
      free(left);
      free(right);
      return i;
    }
  i = abAgregate(a, f);
  res = *i;
  free(i);
  return res;
}

/*-------------------------------------------------------------*/

unsigned int abSize(abP a)
{
  unsigned int *i;
  unsigned int res;
  void* f(void* data, void* left, void* right)
    {
      unsigned int* i = (unsigned int*)easyMalloc(sizeof(unsigned int));
      if (data == NULL)
	*i = 0;
      else
	*i = 1 + *(unsigned int*)left + *(unsigned int*)right;
      free(left);
      free(right);
      return i;
    }
  i = abAgregate(a, f);
  res = *i;
  free(i);
  return res;
}

/*-------------------------------------------------------------*/

void abPrint(abP a, void(*print)(void*))
{
  if (a != NULL)
    {
      printf("(");
      abPrint(a->left, print);
      printf(", ");
      print(a->data);
      printf(", ");
      abPrint(a->right, print);
      printf(")");
    }
}

/*-------------------------------------------------------------*/

int puissance(int b, int n)
{
  if (n == 0)
    return 1;
  return b * puissance(b, n-1);
}

int* makeInt(int i)
{
  int* p = (int*)easyMalloc(sizeof(int));
  *p = i;
  return p;
}

abP intTree(int inf, int sup)
{
  int mid = (inf + sup)/2;
  if (inf > sup)
    return NULL;
  return abMake(makeInt(mid), 
		intTree(inf, mid - 1), 		
		intTree(mid + 1, sup));
}

void destroyInt(void* i)
{
  free(i);
}

void printInt(void* i)
{
  printf("%d", *(int*)i);
}

int main()
{
  abP a = intTree(0, puissance(2, 5));
  abPrint(a, printInt);
  printf("\n");
  abDestroy(a, destroyInt);
  return 0;
}
