#include"parser.h"

int isAnOperator(char c)
{
  return c==PLUS || c==MINUS || c==TIMES || c==DIV || c==EXP;
}

/***********************************************************/

int isANumber(char c)
{
  return c>= '0' && c <= '9';
}

/***********************************************************/

int isAVariable(char c)
{
  return c == VARIABLE;
}

/***********************************************************/

int isLn(char c)
{
  return c == LN;
}

/***********************************************************/

int intOfChar(char a)
{
  return a - '0';
}

/***********************************************************/

int intOfStrAux(char* c, int acc)
{
  if (!isANumber(*c))
    return acc;
  return intOfStrAux(c + 1, intOfChar(*c) + 10 * acc);
}


/***********************************************************/

int intOfStr(char* c)
{
  return intOfStrAux(c, 0);
}

/***********************************************************/

stackTree* addNodeST(stackTree* st, char* s)
{
  if (s==NULL)
    return st;
  if (isAVariable(*s))
    return addVariableST(st);
  if (isAnOperator(*s))
    return addOperatorST(st, *s);
  if (isLn(*s))
    return addLnST(st, *s);
  if (isANumber(*s))
    return addConstantST(st, intOfStr(s));
  printf("Parse error in expression %s.\n", s);
  return NULL;
}

/***********************************************************/

int isADelimitor(char c)
{
  return !(isLn(c) || isAVariable(c) || isANumber(c) || isAnOperator(c));
}

/***********************************************************/

char* findDelimitor(char* str)
{
  while(!isADelimitor(*str))
    {
      str++;
    }
  return str;
}

/***********************************************************/

char* skipDelimitors(char* str)
{
  while(isADelimitor(*str))
    {
      str++;
    }
  return str;
}

/***********************************************************/

int splitStr(char* str, char** adresses)
{
  int res = 1;
  if (*str == 0)
    return 0;
  *adresses = str;
  adresses++;
  *adresses = str + 1;
  while(**adresses != 0)
    {
      *adresses = skipDelimitors(findDelimitor(*adresses));
      *(adresses + 1) = *adresses + 1;
      adresses++;
      res ++;
    }
  return res;
}

/***********************************************************/

sTree* parseExpression(char* exp)
{
  char* adresses[MAX_SIZE];
  int i, nbTokens = splitStr(exp, adresses);
  stackTree* st = NULL;
  sTree* r;
  for(i = 0 ; i < nbTokens ; i++)
    st = addNodeST(st, *(adresses + i));
  r = topST(st);
  st = popST(st);
  if (!isEmptyST(st))
    {
      printf("error while parsing %s\n", exp);
      destroyST(st);
    }
  return r;
}


