#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include"sTree.h"

/***********************************************************/

/*
  Cree un noeud de l'arbre syntaxique.
*/

sTree* createSTree(char type, int data, sTree* left, sTree* right)
{
  sTree* s = (sTree*)malloc(sizeof(sTree));
  if (s == NULL)
    {
      printf("no memory left...\n");
      exit(0);
    }
  s->type = type;
  s->data = data;
  s->left = left;
  s->right = right;
  return s;
}

/***********************************************************/

sTree* createConstantSTree(int constant)
{
  return createSTree(CONSTANT, constant, NULL, NULL);
}

/***********************************************************/

sTree* createOperatorSTree(int operator, sTree* left, sTree* right)
{
  return createSTree(OPERATOR, operator, left, right);
}

/***********************************************************/

sTree* createVariableSTree()
{
  return createSTree(VARIABLE, 0, NULL, NULL);
}

/***********************************************************/

sTree* createLnSTree(sTree* arg)
{
  return createSTree(LN, LN, arg, NULL);
}

/***********************************************************/

void printSTree(sTree* s)
{
  if (s != NULL)
    {
      switch(s->type)
	{
	case CONSTANT : 
	  printf("%d", s->data);
	  break;
	case VARIABLE : 
	  printf("%c", VARIABLE);
	  break;
	case LN : 
	  printf("(ln ");	
	  printSTree(s->left);
	  printf(")");
	  break;
	default : 
	  printf("(");	
	  printSTree(s->left);
	  printf("%c", s->data);
	  printSTree(s->right);
	  printf(")");
	}      
    }
}

/***********************************************************/

void destroySTree(sTree* s)
{
  if (s != NULL)
    {
      destroySTree(s->left);
      destroySTree(s->right);
      free(s);
    }
}

/***********************************************************/

float evaluateSTree(sTree* s, float x)
{
  if (s == NULL)
    {
      printf("STree can't be empty !");
      exit(0);
    }
      switch(s->type)
	{
	case CONSTANT : 
	  return s->data;
	case VARIABLE : 
	  return x;
	default : 
	  switch(s->data)
	    {
	    case PLUS : 
	      return evaluateSTree(s->left, x) + evaluateSTree(s->right, x);
	    case MINUS : 
	      return evaluateSTree(s->left, x) - evaluateSTree(s->right, x);
	    case TIMES : 
	      return evaluateSTree(s->left, x) * evaluateSTree(s->right, x);
	    case DIV : 
	      return evaluateSTree(s->left, x) / evaluateSTree(s->right, x);
	    case LN : 
	      return log(evaluateSTree(s->left, x));
	    default:
	      return pow(evaluateSTree(s->left, x), 
			      evaluateSTree(s->right, x));
	    }
	}      
}

/***********************************************************/

sTree* copySTree(sTree* s)
{
  if (s == NULL)
    return NULL;
  return createSTree(s->type, s->data, 
		     copySTree(s->left), copySTree(s->right)); 
}

/***********************************************************/

sTree* derivateProduct(sTree* s, int sign)
{
  sTree* left = createOperatorSTree(TIMES,
				    derivateSTree(s->left),
				    copySTree(s->right));
  sTree* right = createOperatorSTree(TIMES,
				     copySTree(s->left),
				     derivateSTree(s->right));  
  return createOperatorSTree(sign, left, right);     
}

/***********************************************************/

sTree* derivateRationnal(sTree* s)
{
  sTree* numerator = derivateProduct(s, MINUS);
  sTree* denominator = createOperatorSTree(EXP, copySTree(s->right), 
					   createConstantSTree(2));     
  return createOperatorSTree(DIV, numerator, denominator);
}

/***********************************************************/

sTree* derivateExp(sTree* s)
{
  sTree* tmp, *left;
  tmp = createOperatorSTree(TIMES,
			    copySTree(s->right), 
			    createLnSTree(copySTree(s->left)));
  left = derivateSTree(tmp);
  destroySTree(tmp);
  return createOperatorSTree(TIMES,
		      left, 
		      copySTree(s));
}

/***********************************************************/

sTree* derivateLn(sTree* s)
{
  sTree* numerator = derivateSTree(s->left);
  sTree* denominator = copySTree(s->left);
  return createOperatorSTree(DIV, numerator, denominator);
}

/***********************************************************/

sTree* derivateSTree(sTree* s)
{
  if (s == NULL)
    return NULL;
  switch(s->type)
    {
    case CONSTANT : 
      return createConstantSTree(0); 
    case VARIABLE : 
      return createConstantSTree(1); 
    default :
      switch(s->data)
	{
	case PLUS : 
	  return createOperatorSTree(PLUS, 
				     derivateSTree(s->left), 
				     derivateSTree(s->right));
	case MINUS : 
	  return createOperatorSTree(MINUS, 
				     derivateSTree(s->left), 
				     derivateSTree(s->right));
	case TIMES : 
	  return derivateProduct(s, PLUS);     
	case DIV : 
	  return derivateRationnal(s);     
	case LN : 
	  return derivateLn(s);     
	default:
	  return derivateExp(s);
	}
    }
}

/***********************************************************/

sTree* nDerivateSTree(sTree* s, int n)
{
  if (n == 0)
    return copySTree(s);
  if (n == 1)
    return derivateSTree(s);
  sTree* tmp = nDerivateSTree(s, n - 1);
  s = derivateSTree(tmp);
  destroySTree(tmp);
  return s;
}

/***********************************************************/

float integrateSTree(sTree* s, float a, float b, int nbTrapezes)
{
  if (a > b)
    return - integrateSTree(s, b, a, nbTrapezes);
  float sum, step = (b - a) / nbTrapezes, x = a + step;
  sum = (evaluateSTree(s, a) + evaluateSTree(s, b))/2;
  while(x < b)
    {
      sum += evaluateSTree(s, x);
      x+=step;
    }
  return sum*step;
}








