#include"tree.h"

/************************************************/

/*
  La plupart des fonctionalites sont implementees dans 
  deux fonctions, une dont le suffixe est Tree, et une 
  dont le suffixe est Sons. Les deux fonctions 
  sont generalement mutuellement recursives, ce qui 
  signifie que chacune invoque l'autre.
 */

/************************************************/

tree* createTree(int data, treeList* sons)
{
  tree* t = (tree*)malloc(sizeof(tree));
  if (t == NULL)
    {
      printf("No memory available\n");
      exit(0);
    }
  t->data = data;
  t->sons = sons;
  return t;
}

/************************************************/

treeList* createSon(tree* data, treeList* next)
{
  treeList* s = (treeList*)malloc(sizeof(treeList));
  if (s == NULL)
    {
      printf("No memory available\n");
      exit(0);
    }
  s->data = data;
  s->next = next;
  return s;
}

/************************************************/

void printSons(treeList* l)
{
  if (l != NULL)
    {
      printTree(l->data);
      if (l->next != NULL)
	{
	  printf(", ");
	  printSons(l->next);
	}
    }
}

/************************************************/

void printTree(tree* t)
{
  if (t != NULL)
    {
      printf("(%d [", t->data);
      printSons(t->sons);
      printf("])");
    }
  else
    printf("X");
}

/************************************************/

treeList* randomSonsAux(int depht, int arity, int nbSons)
{
  if (nbSons == 0)
    return NULL;
  return createSon(randomTree(depht, arity),
		   randomSonsAux(depht, arity, nbSons - 1)
		   );
}

/************************************************/

treeList* randomSons(int depht, int arity)
{
  return randomSonsAux(depht, arity, arity);
}

/************************************************/

tree* randomTree(int depht, int arity)
{
  if (depht < 0)
    return NULL;
  return createTree(rand()%MOD, 
		  randomSons(depht - 1, arity)
		  );
}

/************************************************/

void destroySons(treeList* s)
{
  if (s != NULL)
    {
      destroySons(s->next);
      destroyTree(s->data);
      free(s);
    }
}

/************************************************/

void destroyTree(tree* t)
{
  if (t != NULL)
    {
      destroySons(t->sons);
      free(t);
    }
}

/************************************************/

int countSons(treeList* s)
{
  if (s == NULL)
    return 0;
  return countTree(s->data) + countSons(s->next);
}

/************************************************/

int countTree(tree* t)
{
  if (t == NULL)
    return 0;
  return 1 + countSons(t->sons);
}

/************************************************/

int sumSons(treeList* s)
{
  if (s == NULL)
    return 0;
  return sumTree(s->data) + sumSons(s->next);
}

/************************************************/

int sumTree(tree* t)
{
  if (t == NULL)
    return 0;
  return t->data + sumSons(t->sons);
}

/************************************************/

int heightSons(treeList* s)
{
  if (s == NULL)
    return -1;
  return MAX(heightTree(s->data), heightSons(s->next));
}

/************************************************/

int heightTree(tree* t)
{
  if (t == NULL)
    return -1;
  return 1 + heightSons(t->sons);
}

/************************************************/

int findSons(treeList* s, int x)
{
  if (s == NULL)
    return 0;
  return findTree(s->data, x) || findSons(s->next, x);
}

/************************************************/

int findTree(tree* t, int x)
{
  if (t == NULL)
    return 0;
  return (x == t->data) || findSons(t->sons, x);
}

/************************************************/

int nbXSons(treeList* s, int x)
{
  if (s == NULL)
    return 0;
  return nbXTree(s->data, x) + nbXSons(s->next, x);
}

/************************************************/

int nbXTree(tree* t, int x)
{
  if (t == NULL)
    return 0;
  return (x == t->data) + nbXSons(t->sons, x);
}

/************************************************/

treeList* copySons(treeList* s)
{
  if (s == NULL)
    return NULL;
  return createSon(copyTree(s->data), copySons(s->next));
}

/************************************************/

tree* copyTree(tree* t)
{
  if (t == NULL)
    return NULL;
  return createTree(t->data, copySons(t->sons));
}

/************************************************/

tree* treeOfBTree(bTree* t)
{
  if (t == NULL)
    return NULL;
  return createTree(t->data, 
		    createSon(treeOfBTree(t->left), 
			      createSon(treeOfBTree(t->right), NULL)
			      )
		  );
}

/************************************************/

ll* llOfTreeAux(tree* t, ll* acc);

ll* llOfSons(treeList* t, ll* acc)
{
  if (t == NULL)
    return acc;
  acc = llOfSons(t->next, acc);
  return llOfTreeAux(t->data, acc);
}

/************************************************/

ll* llOfTreeAux(tree* t, ll* acc)
{
  if (t == NULL)
    return acc;
  acc = llOfSons(t->sons, acc);
  return createLL(t->data, acc);
}

/************************************************/

ll* llOfTree(tree* t)
{
  return llOfTreeAux(t, NULL);
}

/************************************************/

/*
  Concatene les deux listes l1 et l2.
*/

treeList* concatSons(treeList* l1, treeList* l2)
{
  if (l2 == NULL)
    return l1;
  if (l1 == NULL)
    return l2;
  l1->next = concatSons(l1->next, l2);
  return l1;
}

/************************************************/

tree* destroyRootTree(tree* t)
{
  if (t == NULL || t->sons == NULL)
    return NULL;
  tree* newRoot = t->sons->data;
  newRoot->sons = concatSons(newRoot->sons, t->sons->next);
  free(t);
  return newRoot;
}

/************************************************/

int maxDephtSons(treeList* s, int x)
{
  if (s == NULL)
    return -1;
  return MAX(maxDephtTree(s->data, x), 
	     maxDephtSons(s->next, x)
	     );
}

/************************************************/

int maxDephtTree(tree* t, int x)
{
  if (t == NULL)
    return -1;
  int tmp = maxDephtSons(t->sons, x);
  if (tmp != -1)
    return tmp + 1 ;
  return (t->data == x) - 1;
}

/************************************************/

treeList* deleteXSons(treeList* s, int x)
{
  if (s == NULL)
    return NULL;
  s->data = deleteXTree(s->data, x);
  s->next = deleteXSons(s->next, x);
  return s;
}

/************************************************/

tree* deleteXTree(tree* t, int x)
{
  if (t == NULL)
    return NULL;
  t->sons = deleteXSons(t->sons, x);
  return (t->data == x) ? destroyRootTree(t): t;
}

/************************************************/

treeList* deleteNullSons(treeList* s)
{
  if (s == NULL)
    return NULL;
  s->next = deleteNullSons(s->next);
  s->data = deleteNullTree(s->data);
  if (s->data != NULL)
    return s;
  treeList* tmp = s->next;
  free(s);
  return tmp;
  
}

/************************************************/

tree* deleteNullTree(tree* t)
{
  if (t == NULL)
    return NULL;
  t->sons = deleteNullSons(t->sons);
  return t;
}
