#ifndef B_TREE_H
#define B_TREE_H

#include"ll.h"

#define MAX(i, j) ((i < j) ? j : i)
#define MOD 20

typedef struct bTreeS
{
  int data;
  struct bTreeS* left;
  struct bTreeS* right;
}bTree;

/*
  Cree un noeud et le retourne.
*/

bTree* createBT(bTree* left, int data, bTree* right);
 
/*
  Detruit l'arbre de racine de root.
*/

void destroyBT(bTree* root);

/*
  Affiche l'arbre de racine root.
*/

void printBT(bTree* root);

/*
  Retourne une liste chainee contenant les donnees de 
  l'arbre de racine root.
*/

ll* llOfBT(bTree* root);

/*
  Retourne une liste chainee contenant les donnees de 
  l'arbre de racine root sans que la meme donnee apparaisse
  deux fois.
*/

ll* distinctllOfBT(bTree* root);

/*
  Retourne un arbre de hauteur depht contenant des 
  valeurs aleatoires. 
*/

bTree* randomBT(int depht);

/*
  Retourne la hauteur de l'arbre de racine root.
*/

int heightBT(bTree* root);

/*
  Retourne la somme des elements de l'arbre de racine 
  root.
*/

int sumBT(bTree* root);

/*
  Retourne vrai si l'element x se trouve dans l'arbre de 
  racine root.
*/

int findBT(bTree* root, int x);

/*
  Retourne le nombre d'occurrences de x dans l'arbre de racine 
  root.
*/

int nbXBT(bTree* root, int x);

/*
  Retourne une copie de l'arbre de racine root.
*/

bTree* copyBT(bTree* root);

/*
  Retourne vrai si root est une feuille.
*/

int isALeaf(bTree* root);

/*
  Retourne la plus grande valeur de l'arbre de 
  racine root.
*/

int maxBT(bTree* root);

/*
  Retourne l'arbre de racine root auquel toutes les feuilles
  ont ete supprimees.
*/

bTree* destroyLeavesBT(bTree* root);

/*
  Retourne un arbre de hauteur n contenant les entiers 
  1, 2, ..., 2^n - 1 dans l'ordre infixe.
*/

bTree* integersBT(int n);

/*
  Detruit la racine de l'arbre de racine root. La racine est 
  remplacee par son fils gauche si elle en a un, sinon par 
  son fils droit. La racine du nouvel arbre est retournee.
*/

bTree* destroyRootBT(bTree* root);

/*
  Retourne la profondeur de l'occurence de x de profondeur 
  maximale dans l'arbre de racine root. -1 si x n'est 
  pas dans cet arbre.
*/

int maxDephtBT(bTree* root, int x);

/*
  Supprime toutes les occurences de x dans l'arbre de racine 
  root. Retourne la racine du nouvel arbre.
*/

bTree* deleteXBT(bTree* root, int x);

/*
  Supprime de l'arbre de racine root tous les noeuds ayant 
  comme descendant un noeud de meme valeur.
*/

bTree* purgeBT(bTree* root);

/*
  Retourne les valeurs se trouvant sur le chemin entre la racine 
  root et un noeud de valeur x. Retourne une liste vide si x 
  n'a pas d'occurrence dans cet arbre.
*/

ll* findPathBT(bTree* root, int x);

#endif
