#ifndef G_T_H

#include<stdio.h>
#include<stdlib.h>


#define G_T_H

/*
  Noeud ou feuille d'un arbre binaire
*/

typedef struct ab
{
  /*
    Pointeur vers la donnee
   */
  void* data;

  /*
    Pointeur vers le sous-arbre gauche
   */
  struct ab * left;

  /*
    Pointeur vers le sous-arbre droit
   */
  struct ab * right;
}ab, *abP;


/*
  Construit et retourne un arbre binaire a partir des pointeurs 
  vers la donnee, le sous-arbre gauche et le sous-arbre droit.
 */
abP abMake(void* data, abP left, abP right);

/*
  Construit une donnee a partir d'appels de la forme 
  	f(donne, f(sous-arbre gauche), f(sous-arbre droit))
  Cette donnee peut etre le nombre de noeud de l'arbre si 
  	f(a, b, c) = a + b + c
  un autre arbre, etc. Si a est vide, les triplet (NULL, NULL, NULL)
  est passe en parametre a f.
 */
void* abAgregate(abP a, 
		 void* (*f)(void* data, 
			    void* leftImage, 
			    void* rightImage));

/*
  Retourne un arbre contenant les images par f des 
  donnees de abP. Exemple d'utilisation de la fonction 
  abAgregate.
*/
abP abMap(abP a, void* (*f)(void*));

/*
  Applique la fonction f a toutes les donnes de a, f est d'abord applique 
  aux donnees du sous-arbre gauche, puis a la donnee, puis aux donnees du 
  sous-arbre droit.
 */
void abInfixApply(abP a, void (*f)(void*));

/*
  Detruit l'arbre a, utilise destroyData pour detruire les 
  donnees.
*/
void abDestroy(abP a, void (*destroyData)(void*));

/*
  Retourne la hauteur de l'arbre a, cette fonction est un exemple 
  d'utilisation de la fonction abAgregate.
*/
unsigned int abHeight(abP a);

/*
  Retourne le nombre de noeuds de a, autre exemple d'utilisation 
  de la fonction agregate.
*/
unsigned int abSize(abP a);

#endif
