#ifndef TREE_H
#define TREE_H

#include"bTree.h"
#include"ll.h"

struct tree;

typedef struct treeListS
{
  struct tree* data;
  struct treeListS* next;
}treeList;

typedef struct tree
{
  int data;
  treeList* sons;
}tree;

/*
  Cree un noeud et le retourne.
*/

tree* createTree(int data, treeList* sons);

/*
  Affiche toutes les valeurs de t, racine en premier et feuilles 
  en dernier.
  
*/

void printTree(tree* t);

/*
  Retourne un arbre contenant des valeurs aleatoires. Cet arbre 
  est de profondeur depht et chaque noeud a arity fils.
  
*/

tree* randomTree(int depth, int arity);

/*
  Detruit l'arbre de racine t.
*/

void destroyTree(tree* t);

/*
  Retourne le nombre de noeuds non nuls de l'arbre de racine t.
*/

int countTree(tree* t);

/*
  Retourne la somme des valeurs des noeuds de l'arbre 
  de racine t.
*/

int sumTree(tree* t);

/*
  Retourne la hauteur de l'arbre de racine t.
*/

int heightTree(tree* t);

/*
  Retourne vrai si un noeud de t a la valeur x.
*/

int findTree(tree* t, int x);

/*
  Retourne le nombre d'occurrences de x dans t.
*/

int nbXTree(tree* t, int x);

/*
  Retourne une copie de l'arbre de racine t.
*/

tree* copyTree(tree* t);

/*
  Convertit un arbre binaire en arbre n-aire.
*/

tree* treeOfBTree(bTree* t);

/*
  Retourne une liste chainee contenant les valeurs 
  des noeuds de l'arbre de racine t.
*/

ll* llOfTree(tree* t);

/*
  Detruit la racine de l'arbre de racine t, la remplace par son 
  premier fils.
*/

tree* destroyRootTree(tree* t);

/*
  Retourne la profondeur de l'occurrence de x de profondeur maximale 
  dans l'arbre de racine t. -1 si x n'est pas dans t.
*/

int maxDephtTree(tree* t, int x);

/*
  Detruit toutes les occurrences de x dans t.
*/

tree* deleteXTree(tree* t, int x);

/*
  Supprime tous les pointeurs NULL dans les listes de fils de l'arbre 
  t.
*/

tree* deleteNullTree(tree* t);

#endif
