#ifndef PARSER_H
#define PARSER_H

#include"sTree.h"
#include"stackTree.h"

/*
  Retourne vrai si c est un operateur
*/

int isAnOperator(char c);

/*
  Retourne vrai si c est dans l'intervalle '0', ..., '9'
*/

int isANumber(char c);

/*
  Retourne vrai si c est la variable
*/

int isAVariable(char c);

/*
  Retourne vrai si c la lettre representant le logarithme.
*/

int isLn(char c);

/*
  Convertit '0' en 0, '1' en 1, '2' en 2, ...
*/

int intOfChar(char a);

/*
  Convertit en nombre une chaine de caractere.
  Par exemple intOfStr("123") = 123.
*/

int intOfStr(char* c);

/*
  Ajoute dans la pile un arbre syntaxique implementant l'expression
  representee dans *s.
*/

stackTree* addNodeST(stackTree* st, char* s);

/*
  Retourne vrai si c est un delimiteur. Nous considererons comme
  tel tous les caracteres autres que constante, variable, operateur 
  et logarithme.
*/

int isADelimitor(char c);

/*
  Retourne l'adresse du premier delimiteur se trouvant 
  dans str.
*/

char* findDelimitor(char* str);

/*
  Retourne l'adresse du premier caractere de str
  n'etant pas un delimiteur.
*/

char* skipDelimitors(char* str);

/*
  Place dans le tableau *adresses (suppose suffisament grand) les 
  adresses de toutes les chaines sans delimiteurs de str. 
  Retourne le nombre de chaines trouvees.
*/

int splitStr(char* str, char** adresses);

/*
  Retourne un arbre syntaxique representant l'expression 
  stockee en ecriture post-fixe dans exp.
*/

sTree* parseExpression(char* exp);

#endif
