#ifndef S_TREE_H
#define S_TREE_H

#define OPERATOR 1
#define CONSTANT 2
#define VARIABLE 'X'
#define LN 'L'

#define PLUS '+'
#define MINUS '-'
#define TIMES '*'
#define DIV '/'
#define EXP '^'

typedef struct st
{
  char type;
  int data;
  struct st* left;
  struct st* right;
}sTree;

/*
  Cree un noeud representant une constante.
*/

sTree* createConstantSTree(int constant);

/*
  Cree une noeud representant la variable.
*/

sTree* createVariableSTree();

/*
  Cree un noeud representant une operation entre 
  le sous-arbre gauche et le sous-arbre droit.
*/

sTree* createOperatorSTree(int operator, sTree* left, sTree* right);

/*
  Cree un noeud representant le logarithme du sous-arbre arg
  (place dans left).
*/

sTree* createLnSTree(sTree* arg);

/*
  Affiche l'arbre s.
*/

void printSTree(sTree* s);

/*
  Detruit l'arbre s.
*/

void destroySTree(sTree* s);

/*
  Retourne la valeur de l'expression representee par s en substituant 
  x a l'inconnue.
*/

float evaluateSTree(sTree* s, float x);

/*
  Retourne une copie de s.
*/

sTree* copySTree(sTree* s);

/*
  Retourne la derivee de s.
*/

sTree* derivateSTree(sTree* s);

/*
  Retourne la derivee n-eme de s.
*/

sTree* NderivateSTree(sTree* s, int n);

/*
  Calcule avec la methode des trapezes l'integrale
  de l'expression s entre a et b.
*/

float integrateSTree(sTree* s, float a, float b, int nbTrapezes);

#endif
